pub enum BorrowKind {
Shared,
Shallow,
Mut {
kind: MutBorrowKind,
},
}Variants
Data must be immutable and is aliasable.
Shallow
The immediately borrowed place must be immutable, but projections from
it don’t need to be. For example, a shallow borrow of a.b doesn’t
conflict with a mutable borrow of a.b.c.
This is used when lowering matches: when matching on a place we want to ensure that place have the same value from the start of the match until an arm is selected. This prevents this code from compiling:
ⓘ
let mut x = &Some(0);
match *x {
None => (),
Some(_) if { x = &None; false } => (),
Some(_) => (),
}This can’t be a shared borrow because mutably borrowing (*x as Some).0
should not prevent if let None = x { ... }, for example, because the
mutating (*x as Some).0 can’t affect the discriminant of x.
We can also report errors with this kind of borrow differently.
Mut
Data is mutable and not aliasable.
Fields

kind: MutBorrowKindImplementations
Sourceimpl BorrowKind
impl BorrowKind
fn from_hir_mutability(m: Mutability) -> Self
fn from_rustc_mutability(m: Mutability) -> Self
fn from_hir(bk: BorrowKind) -> Self