pub enum MirLowerError<'db> {
Show 24 variants
ConstEvalError(Box<str>, Box<ConstEvalError<'db>>),
LayoutError(LayoutError),
IncompleteExpr,
IncompletePattern,
TraitFunctionDefinition(TraitId, Name),
UnresolvedName(String),
RecordLiteralWithoutPath,
UnresolvedMethod(String),
UnresolvedField,
UnsizedTemporary(StoredTy),
MissingFunctionDefinition(InferBodyId<'db>, ExprId),
HasErrors,
TypeError(&'static str),
NotSupported(String),
ContinueWithoutLoop,
BreakWithoutLoop,
Loop,
ImplementationError(String),
LangItemNotFound,
MutatingRvalue,
UnresolvedLabel,
UnresolvedUpvar(Place),
InaccessibleLocal,
GenericArgNotProvided(GenericParamId, StoredGenericArgs),
}Variants
ConstEvalError(Box<str>, Box<ConstEvalError<'db>>)
LayoutError(LayoutError)
IncompleteExpr
IncompletePattern
TraitFunctionDefinition(TraitId, Name)
Trying to lower a trait function, instead of an implementation
UnresolvedName(String)
RecordLiteralWithoutPath
UnresolvedMethod(String)
UnresolvedField
UnsizedTemporary(StoredTy)
MissingFunctionDefinition(InferBodyId<'db>, ExprId)
HasErrors
TypeError(&'static str)
This should never happen. Type mismatch should catch everything.
NotSupported(String)
ContinueWithoutLoop
BreakWithoutLoop
Loop
ImplementationError(String)
Something that should never happen and is definitely a bug, but we don’t want to panic if it happened
LangItemNotFound
MutatingRvalue
UnresolvedLabel
UnresolvedUpvar(Place)
InaccessibleLocal
GenericArgNotProvided(GenericParamId, StoredGenericArgs)
Implementations
Sourceimpl MirLowerError<'_>
impl MirLowerError<'_>
pub fn pretty_print( &self, f: &mut String, db: &dyn HirDatabase, span_formatter: impl Fn(FileId, TextRange) -> String, display_target: DisplayTarget, ) -> Result<(), Error>
Sourceimpl MirLowerError<'_>
impl MirLowerError<'_>
fn unresolved_path( db: &dyn HirDatabase, p: &Path, display_target: DisplayTarget, owner: ExpressionStoreOwnerId, store: &ExpressionStore, ) -> Self