pub enum MutBorrowKind {
ClosureCapture,
Default,
TwoPhasedBorrow,
}Variants
ClosureCapture
Data must be immutable but not aliasable. This kind of borrow cannot currently be expressed by the user and is used only in implicit closure bindings.
Default
TwoPhasedBorrow
This borrow arose from method-call auto-ref (i.e., adjustment::Adjust::Borrow).