pub struct SwitchTargets {
values: SmallVec<[u128; 1]>,
targets: SmallVec<[BasicBlockId; 2]>,
}Fields
values: SmallVec<[u128; 1]>Possible values. The locations to branch to in each case
are found in the corresponding indices from the targets vector.
targets: SmallVec<[BasicBlockId; 2]>Possible branch sites. The last element of this vector is used for the otherwise branch, so targets.len() == values.len() + 1 should hold.
Implementations
Sourceimpl SwitchTargets
impl SwitchTargets
Sourcepub fn new(
targets: impl Iterator<Item = (u128, BasicBlockId)>,
otherwise: BasicBlockId,
) -> Self
pub fn new( targets: impl Iterator<Item = (u128, BasicBlockId)>, otherwise: BasicBlockId, ) -> Self
Creates switch targets from an iterator of values and target blocks.
The iterator may be empty, in which case the SwitchInt instruction is equivalent to
goto otherwise;.
Sourcepub fn static_if(value: u128, then: BasicBlockId, else_: BasicBlockId) -> Self
pub fn static_if(value: u128, then: BasicBlockId, else_: BasicBlockId) -> Self
Builds a switch targets definition that jumps to then if the tested value equals value,
and to else_ if not.
Sourcepub fn otherwise(&self) -> BasicBlockId
pub fn otherwise(&self) -> BasicBlockId
Returns the fallback target that is jumped to when none of the values match the operand.
Sourcepub fn iter(&self) -> impl Iterator<Item = (u128, BasicBlockId)> + '_
pub fn iter(&self) -> impl Iterator<Item = (u128, BasicBlockId)> + '_
Returns an iterator over the switch targets.
The iterator will yield tuples containing the value and corresponding target to jump to, not
including the otherwise fallback target.
Note that this may yield 0 elements. Only the otherwise branch is mandatory.
Sourcepub fn all_targets(&self) -> &[BasicBlockId] ⓘ
pub fn all_targets(&self) -> &[BasicBlockId] ⓘ
Returns a slice with all possible jump targets (including the fallback target).
Sourcepub fn target_for_value(&self, value: u128) -> BasicBlockId
pub fn target_for_value(&self, value: u128) -> BasicBlockId
Finds the BasicBlock to which this SwitchInt will branch given the
specific value. This cannot fail, as it’ll return the otherwise
branch if there’s not a specific match for the value.